@@ -2,7 +2,6 @@ import os
 from PIL import Image
 import numpy
 from torch import LongTensor, FloatTensor
-from math import sqrt
 import torch
 import torch.backends.cudnn, torch.backends.cuda
 import json
@@ -15,8 +14,6 @@ torch.set_grad_enabled(False)
 torch.set_num_threads(os.cpu_count())
 torch.backends.cudnn.enabled = True
 torch.backends.cudnn.allow_tf32 = True
-torch.backends.cuda.matmul.allow_tf32 = True
-torch.backends.cuda.matmul.allow_fp16_reduced_precision_reduction = True
 
 MIN_DALLE_REPO = 'https://huggingface.co/kuprel/min-dalle/resolve/main/'
 IMAGE_TOKEN_COUNT = 256
@@ -39,7 +36,6 @@ class MinDalle:
         self.is_mega = is_mega
         self.is_reusable = is_reusable
         self.dtype = dtype
-        self.vqgan_dtype = torch.float32 if dtype == torch.float16 else dtype
         self.is_verbose = is_verbose
         self.text_token_count = 64
         self.layer_count = 24 if is_mega else 12
@@ -236,7 +232,8 @@ class MinDalle:
             dtype=torch.float32,
             device=self.device
         )
-        for i in range(IMAGE_TOKEN_COUNT):                
+        for i in range(IMAGE_TOKEN_COUNT):
+            torch.cuda.empty_cache()                
             with torch.cuda.amp.autocast(dtype=self.dtype):
                 image_tokens[i + 1], attention_state = self.decoder.forward(
                     settings=settings,
@@ -247,7 +244,7 @@ class MinDalle:
                     token_index=token_indices[[i]]
                 )
 
-            with torch.cuda.amp.autocast(dtype=self.vqgan_dtype):
+            with torch.cuda.amp.autocast(dtype=torch.float32):
                 if ((i + 1) % 32 == 0 and progressive_outputs) or i + 1 == 256:
                     yield self.image_grid_from_tokens(
                         image_tokens=image_tokens[1:].T,
